
//	standard includes
#include "stdafx.h"
#include	"math.h"

//	my includes
#include "KDEclass.h"
#include "myarrayutils.h"

/*************************************************************************************************/
//------------------------------------------------------------------------------------------------
// PRIVATE FUNCTIONS
//------------------------------------------------------------------------------------------------
/*************************************************************************************************/



/*************************************************************************************************/
/*************************************************************************************************/
/*************************************************************************************************/






/*************************************************************************************************/
//------------------------------------------------------------------------------------------------
// PUBLIC FUNCTIONS
//------------------------------------------------------------------------------------------------
/*************************************************************************************************/

// CONSTRUCTORS
KDEclass::KDEclass()
{

}

/*************************************************************************************************/


// DESTRUCTOR
KDEclass::~KDEclass()
{

}

/*************************************************************************************************/


// compute distance matrix
void KDEclass::computeDistanceMatrix()
{
	vnl_sym_matrix<double> Dsym = ComputeGeodesicDistanceMatrix(covdesc,N);
	D.set_size(N,N);
	D = Dsym.as_matrix();
}


/*************************************************************************************************/

// learn KDE bandwidth
void KDEclass::learnKDEbandwidth()
{
	double old_sigma2 = 0.0;
	sigma2 = 1.1236 * pow(N,0.4);
	double dim = (float) covdesc[0].dim;
	vnl_matrix<double> tmp(1,N,0.0);
	vnl_vector<double> d(1,0.0);

	vnl_sym_matrix<double> G(N,0.0);
	vnl_matrix<double> G1(N,N,0.0);

	


	int iter = 0;
	while ( ( abs(old_sigma2-sigma2)>LEARN_BANDWIDTH_THRESHOLD ) && ( iter<MAX_ITERS_KDE ) )
	{

		old_sigma2 = sigma2; iter++;
		for (int i=0;i<N;i++)
			for (int j=i+1;j<N;j++)
				G(i,j) = pow(2*PI*old_sigma2,-dim/2.0) * exp(-0.5*D(i,j)/old_sigma2);
		G1 = G.as_matrix();
		sigma2 = 0.0;
		for (int i=0;i<N;i++)
		{
			tmp.set_row(0,D.get_row(i));
			d = G1.get_column(i).pre_multiply(tmp) ;
			sigma2 += d(0)/G1.get_column(i).mean();			
		}
		sigma2 /= (N*(N-1)*dim);
	}
}

/*************************************************************************************************/



// compute probability estimate
vnl_vector<double> KDEclass::estimateKDEprobability(CovarianceDescriptor* test, int Ntest)
{
	vnl_vector<double> prob(Ntest,1.0);
	vnl_matrix<double> Dtest = ComputeGeodesicDistanceMatrix(test, Ntest, covdesc, N);
	for (int i=0;i<Ntest;i++)
		for (int j=0;j<N;j++)
			Dtest(i,j) = exp(-0.5*Dtest(i,j)/sigma2);		
	
	for (int i=0;i<Ntest;i++)
		prob(i) = Dtest.get_row(i).mean();

	prob *= pow( 2*PI*sigma2 , -((float)dim)/2.0 );
	return prob;
}



/*************************************************************************************************/

//// compute probability estimate with added data points
//vnl_matrix<double> estimateKDEprobability_augmented(CovarianceDescriptor* test, int Ntest);
//{
//	vnl_matrix<double> prob(Ntest,Ntest,0.0);
//	vnl_matrix<double> Dtest = ComputeGeodesicDistanceMatrix(test, Ntest, covdesc, N);
//	for (int i=0;i<Ntest;i++)
//		for (int j=0;j<N;j++)
//			Dtest(i,j) = exp(-0.5*Dtest(i,j)/sigma2);		
//
//	for (int i=0;i<Ntest;i++)
//		prob(i,i) = Dtest.get_row(i).mean();
//
//	prob *= pow( 2*PI*sigma2 , -((float)dim)/2.0 );
//	return prob;
//}
//


/*************************************************************************************************/