
#ifndef __COVDESC_H__
#define __COVDESC_H__

////////////////////////////////////////////////////////////////////////////////////////////////////
// includes

////////////////////////////////////////////////////////////////////////////////////////////////////

/*************************************************************************************************/
class CovarianceDescriptor
{
	/*************************************************************************************************/
	private:
	/*************************************************************************************************/
		//------------------------------------------------------------------------------------------------
		// PRIVATE VARIABLES
		//------------------------------------------------------------------------------------------------
		

		//------------------------------------------------------------------------------------------------
		// PRIVATE FUNCTIONS
		//------------------------------------------------------------------------------------------------


	/*************************************************************************************************/
	public:
	/*************************************************************************************************/
		//------------------------------------------------------------------------------------------------
		// PUBLIC VARIABLES
		//------------------------------------------------------------------------------------------------


		long int numPix;				// number of pixels in image region

		int dim;						// dimension of covariance descriptor

		vnl_matrix<double> cov;			// covariance descriptor

		vnl_matrix<double> invcov;		// pre-computed inverse of covariance for later use

		vnl_matrix<double> invsqrtcov;	// pre-computed inverse-square-root of covariance for later use

		vnl_symmetric_eigensystem<double>* eig; // symmetric eigensystem of covariance descriptor


		//------------------------------------------------------------------------------------------------
		// PUBLIC FUNCTIONS
		//------------------------------------------------------------------------------------------------
		// CONSTRUCTORS

		// default constructor
		CovarianceDescriptor();

		// empty constructor
		CovarianceDescriptor(int d);

		// given the non-centered ( d x N )data matrix with with N points as column vectors
		CovarianceDescriptor(vnl_matrix<double> data);

		// directly given the ( d x d ) covariance matrix
		CovarianceDescriptor(vnl_matrix<double> C, long int N);


		// DESTRUCTOR
		~CovarianceDescriptor();


		// given the non-centered ( d x N )data matrix with with N points as column vectors
		void ComputeDescriptor(vnl_matrix<double> data);

		// directly given the ( d x d ) covariance matrix
		void ComputeDescriptor(vnl_matrix<double> C, long int N);
		
		// directly given the vectorization of the ( d x d ) covariance matrix
		void ComputeDescriptor(vnl_vector<double> c, int dims);

		// Riemannian metric on tangent space of X
		double ComputeRiemannianMetric(vnl_matrix<double> Y, vnl_matrix<double> Z);

		// Exponential map associated to Riemannian metric
		vnl_matrix<double> ExponentialMap(vnl_matrix<double> Y);

		// Logarithmic map associated to Riemannian metric
		vnl_matrix<double> LogarithmicMap(vnl_matrix<double> Y);

		// Geodesic distance squared to another covariance descriptor
		double GeodesicDistance(CovarianceDescriptor Y);



	/*************************************************************************************************/
};


/*************************************************************************************************/

// Computes the squared geodesic distance between all pairs of descriptors - matrices have to be symmetric and PSD
vnl_sym_matrix<double> ComputeGeodesicDistanceMatrix(CovarianceDescriptor* X, int lenX);	

// Computes the squared geodesic distance between all pairs of descriptors - matrices have to be symmetric and PSD
vnl_matrix<double> ComputeGeodesicDistanceMatrix(CovarianceDescriptor* X, int lenX, CovarianceDescriptor* Y, int lenY);

/*************************************************************************************************/



#endif