
#ifndef		__COVMATUTILITIES_H__
#define		__COVMATUTILITIES_H__



//using namespace std;




////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Exponential map in Riemannian geometry
vnl_matrix<double> MatExp(const vnl_matrix<double> &Y, const vnl_matrix<double> &X)
{
	// Exp Y with respect to X
	vnl_symmetric_eigensystem<double> eig(X);
	vnl_matrix<double> invsqrt = eig.inverse_square_root();
	vnl_matrix<double> matsqrt = eig.square_root();
	vnl_matrix<double> expm = matsqrt * vnl_matrix_exp(invsqrt*Y*invsqrt) * matsqrt;
	return expm;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Log map in Riemannian geometry
vnl_matrix<double> MatLog(const vnl_matrix<double> &Y, const vnl_matrix<double> &X)
{
	//Log Y with respect to X
	vnl_symmetric_eigensystem<double> eig(X);
	vnl_matrix<double> invsqrt = eig.inverse_square_root();
	vnl_matrix<double> matsqrt = eig.square_root();
	vnl_symmetric_eigensystem<double> eigGen(invsqrt*Y*invsqrt);
	for (unsigned int i=0; i<Y.cols(); ++i)
		eigGen.D(i,i) = log(eigGen.D(i,i));
	vnl_matrix<double> logm = eigGen.recompose();
	return (matsqrt*logm*matsqrt);

}
////////////////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Computes the geodesic distance between two given matrices - matrices have to be symmetric and PSD
double GeodesicDistance(const vnl_matrix<double> &Y, const vnl_matrix<double> &X)
{	
	// Geodesic distance
	vnl_symmetric_eigensystem<double> eigX(X);
	//vnl_matrix<double> ans = eig.inverse_square_root()*Y*eig.inverse_square_root();	
	vnl_symmetric_eigensystem<double> genEig( eigX.inverse_square_root()*Y*eigX.inverse_square_root() );
	double dist = 0;	
	for (unsigned int i=0; i<Y.cols(); ++i)
		dist += pow(log(genEig.D(i,i)),2.0);
	return sqrt(dist);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Computes the geodesic distance between all pairs in std::vector of matrices - matrices have to be symmetric and PSD
vnl_sym_matrix<double> ComputeGeodesicDistance(vector<vnl_matrix<double>> X)
{
	
	vnl_sym_matrix<double> D; 
	int N = X.size();
	D.set_size(N);
	vnl_matrix<double> invsqrt;	
	int d = X[0].rows();
	invsqrt.set_size(d,d);
	double dist = 0;
	for (int i=0;i<N;i++){
		vnl_symmetric_eigensystem<double> eigX(X[i]);
		invsqrt = eigX.inverse_square_root();
		for (int j=i+1;j<N;j++){
			vnl_symmetric_eigensystem<double> genEig( invsqrt * X[j] * invsqrt );			
			for (int k=0;k<d;k++)
				D(i,j) += pow(log(genEig.D(k,k)),2.0);
		}
	}
	return D;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////



#endif