
#include "stdafx.h"
#include "generalutils.h"

/*	SIMPLE MATRIX INITIALIZATIONS	*/
void setOneMask(IplImage* A)
{
    CvRect aroi = cvGetImageROI( A );
    im_itr<uchar> a(A, aroi.x, aroi.y, aroi.width, aroi.height);
    
    while( !a )
    {
        *a = 255;
        ++a;
    }
}

/*	MATRIX TYPE CONVERSIONS	*/
void convert8Uto32F	( IplImage* src8u , IplImage* dst32f)
{
    CvRect sroi = cvGetImageROI( src8u );
    CvRect droi = cvGetImageROI( dst32f );
    
    im_itr<uchar> src( src8u, sroi.x, sroi.y, sroi.width, sroi.height );
    im_itr<float> dst( dst32f, droi.x, droi.y, droi.width, droi.height );
    
    while( !src && !dst )
    {
        *dst = float(*src) / 255;
        ++dst; ++src;
    }
    
}
void convert32Fto8U	( IplImage* src32f, IplImage* dst8u )
{
    CvRect sroi = cvGetImageROI( src32f );
    CvRect droi = cvGetImageROI( dst8u );
    
    im_itr<float> src( src32f, sroi.x, sroi.y, sroi.width, sroi.height );
    im_itr<uchar> dst( dst8u, droi.x, droi.y, droi.width, droi.height );
    
    while( !src && !dst )
    {
        *dst = uchar((*src) * 255);
        ++dst; ++src;
    }
    
}

void convert64Fto8U ( IplImage* src64f, IplImage* dst8u )
{
    CvRect sroi = cvGetImageROI( src64f );
    CvRect droi = cvGetImageROI( dst8u );
    
    im_itr<double> src( src64f, sroi.x, sroi.y, sroi.width, sroi.height );
    im_itr<uchar> dst( dst8u, droi.x, droi.y, droi.width, droi.height );
    
    while( !src && !dst )
    {
        *dst = uchar((*src) * 255);
        ++dst; ++src;
    }
}
/*	SIMPLE FILTER INITIALIZATIONS	*/
void createGaussianWeights( IplImage* gMat, float sig )
{
    bool iscolor = ( gMat->nChannels > 1 );
    CvRect groi = cvGetImageROI( gMat );
    im_itr<float> g(gMat, groi.x, groi.y, groi.width, groi.height);
    int mn_x = (groi.width/2 + 0.5);
    int mn_y = (groi.height/2 + 0.5);
    
    while( !g )
    {
        int y = g.row() - groi.y;
        int x = g.column() - groi.x;
        float xmean= -(x-mn_x)*(x-mn_x), ymean= -(y-mn_y)*(y-mn_y), sig2=sig*sig;
        
        float val = exp((xmean + ymean)/(2*sig2*CV_PI*groi.height*groi.width));
        *g = val;
        if( iscolor )
        {
            ++g;	*g = val;
            ++g;	*g = val;
        }
        ++g;
    }
}

/*	DISPLAY A GIVEN IMAGE AND WAIT	*/
void displayImage( IplImage* img, char* wname, int waitTimeInMiliSec )
{
    if( img->depth == IPL_DEPTH_32F )
    {
        IplImage* tp00 = cvCreateImage( cvGetSize( img ), IPL_DEPTH_8U, img->nChannels );
        convert32Fto8U( img, tp00 );
        displayImage( tp00, wname, waitTimeInMiliSec );
    }
    else
    {
        if( wname != 0 )	
		{
            cvNamedWindow( wname , 1 );
            cvShowImage( wname, img );
			if (waitTimeInMiliSec == 0)
			{
				cout << "Press any key..." << endl;
			}
            cvWaitKey( waitTimeInMiliSec );
            cvDestroyWindow( wname );
        }
        else				
		{
            cvNamedWindow( "Display" , 1 );
            cvShowImage( "Display", img );
			if (waitTimeInMiliSec == 0)
			{
				cout << "Press any key..." << endl;
			}
            cvWaitKey( waitTimeInMiliSec );
            cvDestroyWindow( "Display" );
        }
    }
}

void displayMarkedBoundary( IplImage* img, CvRect bound, char* wname, int waitTimeInMiliSec )
{
    IplImage* bMask = cvCreateImage( cvGetSize(img), IPL_DEPTH_8U, 1 );
    cvZero( bMask );
    CvRect outer = cvRect( max(0,bound.x-1),max(0,bound.y-1),
    min(img->width,bound.width+2),min(img->height,bound.height+2) );
    cvSetImageROI( bMask, outer );
    setOneMask( bMask );
    cvResetImageROI( bMask );
    cvSetImageROI( bMask, bound );
    cvSet( bMask, cvScalar(0) );
    cvResetImageROI( bMask );
    IplImage* im2show = cvCloneImage( img );
    cvSet( im2show, cvScalar(255), bMask );
    cvReleaseImage( &bMask );
    displayImage( im2show, wname, waitTimeInMiliSec );
    cvReleaseImage( &im2show );
}

/*	SPLIT IMAGES INTO CHANNELS	*/
void splitChannels( deque<IplImage*> &src, deque<IplImage*> &dst )
{
    int nc = (src[0])->nChannels;
    
    for(int ni=0; ni < src.size(); ++ni)
    {
        dst[ni*nc] = cvCreateImage( cvGetSize(src[ni]), (src[ni])->depth, 1 );
        dst[ni*nc + 1] = cvCreateImage( cvGetSize(src[ni]), (src[ni])->depth, 1 );
        dst[ni*nc + 2] = cvCreateImage( cvGetSize(src[ni]), (src[ni])->depth, 1 );
        cvSplit( src[ni], dst[ni*nc], dst[ni*nc + 1], dst[ni*nc + 2], 0 );
    }
}

/*	MERGE IMAGES INTO CHANNELS	*/
void mergeChannels( deque<IplImage*> &src, deque<IplImage*> &dst )
{
    int nc = dst[0]->nChannels;
    
    for(int ni=0; ni < src.size(); ++ni)
	{
        cvMerge( src[ni*nc], src[ni*nc + 1], src[ni*nc + 2], 0, dst[ni] );
	}
    
}

/*	GET BORDER MASK	*/
IplImage* getBorderMask( IplImage* mask, int param )
{
    IplImage* border = cvCloneImage( mask );
    IplImage* buffer = cvCloneImage( mask );
    
    if( param == BORDER_ZERO )	cvErode(mask, buffer);
    else						cvDilate(mask, buffer);
    
    cvXor( mask, buffer, border );
    cvReleaseImage( &buffer );
    
    return	border;
}

/*	CONVERT BGR (UCHAR) TO rgs	*/
void convertBGRtoRGS( IplImage* BGR, IplImage* RGS )
{
    im_itr<uchar> bgr(BGR);
    im_itr<float> rgs(RGS);
    
    while( (!bgr) && (!rgs) )
    {
        uchar B = *bgr;
        ++bgr;
        uchar G = *bgr;
        ++bgr;
        uchar R = *bgr;
        ++bgr;
        
        *rgs = float( R ) / float( R + G + B + 1.0);
        ++rgs;
        *rgs = float( G ) / float( R + G + B + 1.0);
        ++rgs;
        *rgs = float( R + G + B ) / (3.0*255.0);
        ++rgs;
    }
}

IplImage* convertGRAYtoBGR( IplImage* Gray )
{
    if( (Gray->nChannels > 1) || (Gray==0))	return	0;
    
    IplImage* bgr = cvCreateImage( cvGetSize(Gray),Gray->depth,3 );
    IplImage* b = cvCloneImage( Gray );
    IplImage* g = cvCloneImage( Gray );
    IplImage* r = cvCloneImage( Gray );
    cvMerge( b,g,r,0, bgr );
    cvReleaseImage( &b );
    cvReleaseImage( &g );
    cvReleaseImage( &r );
    
    return bgr;
}

void convertGRAYtoBGR( IplImage* Gray, IplImage* bgr )
{
    if( (Gray->nChannels > 1) || (Gray==0))	
	{
		return;
	}
    //IplImage* bgr = cvCreateImage( cvGetSize(Gray),Gray->depth,3 );
    IplImage* b = cvCloneImage( Gray );
    IplImage* g = cvCloneImage( Gray );
    IplImage* r = cvCloneImage( Gray );
    cvMerge( b,g,r,0, bgr );
    cvReleaseImage( &b );
    cvReleaseImage( &g );
    cvReleaseImage( &r );
    
    //return bgr;
}

/*	SAMPLE A GIVEN "UCHAR" MASK UNIFORMLY	*/
void sample_uniformly( IplImage* sample_mask, double percent_sample )
{
    cvZero( sample_mask );
    CvRNG* rng_state = new CvRNG;
    *rng_state = cvRNG( 0xffffffff );
    int numsamp = percent_sample * sample_mask->width * sample_mask->height;
    
    bool allsamples = (numsamp == (sample_mask->width * sample_mask->height));
    if( !allsamples )
    {
        for( int s=0; s < numsamp; ++s )
        {
            int a = int(cvRandReal(rng_state) * sample_mask->width);
            int b = int(cvRandReal(rng_state) * sample_mask->height);
            if( CV_IMAGE_ELEM(sample_mask,uchar,b,a) )
			{
				--s;
			}
            else
			{
				CV_IMAGE_ELEM(sample_mask,uchar,b,a) = 255;
			}
        }
    }
    else
    {
        setOneMask(sample_mask);
    }
}

/* DIVIDE ENTIRE IMAGE BY A SCALAR */
void DivS(IplImage* src32f, double scale)
{
    im_itr<float> src(src32f);
    while( !src )
    {
        *src = (*src / scale);
        ++src;
    }
}

/*	REFINE (MORPHOLOGICALLY) MASK, GIVEN WIN-SIZE
	AND MINIMUM % OF "HIGH" VALUES	*/
void morph_refine_mask( IplImage* mask, 
						CvPoint winsize, double percent_hi)
{
	int wd = winsize.x;
	int ht = winsize.y;
	double hi_thresh = wd*ht*percent_hi/100;
	IplImage* tmp = cvCloneImage( mask );
	cvZero( tmp );

	if( cvCountNonZero( mask ) > hi_thresh )
	{
		for( int mx=0; mx < (mask->width-wd); ++mx )
		{
			for( int my=0; my < (mask->height-ht); ++my )
			{
				CvRect roi = cvRect(mx,my,wd,ht);
				cvSetImageROI( mask, roi );
				double numhi = cvCountNonZero( mask );
				if( numhi > hi_thresh )
				{
					cvSetImageROI( tmp, roi );
					cvCopy( mask, tmp );
					cvResetImageROI( tmp );
				}
				cvResetImageROI( mask );
			}
		}
	}

	cvCopy( tmp, mask );
	cvReleaseImage( &tmp );
}