
//	standard includes
#include "stdafx.h"
#include	"math.h"

//	my includes
#include "imagedescriptor.h"
#include "imgiter.h"


/*************************************************************************************************/
//------------------------------------------------------------------------------------------------
// PRIVATE FUNCTIONS
//------------------------------------------------------------------------------------------------
/*************************************************************************************************/


void ImageDescriptor::MakeNhoodGraph()
{
	vnl_sym_matrix<double> nhoodgraphsym;
	nhoodgraphsym.set_size(numRegions);
	
	nhoodgraph.set_size(numRegions,numRegions);

	float* inum = new float[numRegions];
	IplImage* thismask = cvCreateImage(cvGetSize(masks[0]),IPL_DEPTH_8U,1); cvZero(thismask);

	for (int i=0;i<numRegions;i++)
		inum[i] = (float) cvCountNonZero(masks[i]);

	for (int i=0;i<numRegions;i++)
	{		
		for (int j=i+1;j<numRegions;j++)
		{
			cvAnd(masks[i],masks[j],thismask);
			nhoodgraphsym(i,j) = ((float) cvCountNonZero(thismask))/min(inum[i],inum[j]);
		}
	}
	
	nhoodgraph = nhoodgraphsym.as_matrix();
}



/*************************************************************************************************/

void ImageDescriptor::ComputeGeometrics()
{
	
	eccentricity.set_size(numRegions);
	moments = new CvMoments[numRegions];
	double mu20,mu02,mu11,t;
	for (int i=0;i<numRegions;i++)
	{
		cvMoments(masks[i],&moments[i],1);
		mu20 = cvGetCentralMoment(&moments[i],2,0);
		mu02 = cvGetCentralMoment(&moments[i],0,2);
		mu11 = cvGetCentralMoment(&moments[i],1,1);
		t = sqrt( 4*pow(mu11,2.0) + pow(mu02-mu20,2.0) );
		eccentricity(i) = sqrt(2*t/(mu20+mu02+t));
	}
}


/*************************************************************************************************/



/*************************************************************************************************/
/*************************************************************************************************/
/*************************************************************************************************/






/*************************************************************************************************/
//------------------------------------------------------------------------------------------------
// PUBLIC FUNCTIONS
//------------------------------------------------------------------------------------------------
/*************************************************************************************************/
// CONSTRUCTOR
ImageDescriptor::ImageDescriptor(IplImage** images, IplImage* segMask, int numSegs, int inputDim)
{
	// number of image regions
	numRegions = numSegs;

	// dimensions of covariance descriptor
	dim = inputDim;

	// image size
	imsize = cvGetSize(segMask);

	segmentMask = cvCreateImage(imsize,IPL_DEPTH_8U,1);
	masks = new IplImage*[numRegions];
	numPix = new long int[numRegions];
	covdesc = new CovarianceDescriptor[numRegions];
	vnl_matrix<double>* D = new vnl_matrix<double>[numRegions];


	// create a mask with all pixels unmarked
	IplImage *mysegmask = cvCreateImage(imsize,IPL_DEPTH_8U,1); cvSet(mysegmask,cvScalar(0.0));
	
	double minVal = 0.0, maxVal = 0.0;
	CvPoint minLoc, maxLoc;	

	for (int i=0;i<numRegions;i++)
	{
		// allocate memory for the mask image
		masks[i] = cvCreateImage(imsize,IPL_DEPTH_8U,1);

		// find the next unmarked pixel in mysegmask 
		cvMinMaxLoc(mysegmask,&minVal,&maxVal,&minLoc,&maxLoc);

		// compare the segmentation mask with this unmarked pixel's value
		cvCmpS(segMask,CV_IMAGE_ELEM(segMask,float,minLoc.y,minLoc.x),masks[i],CV_CMP_EQ);

		// mark all pixels in this image region
		cvOr(mysegmask,masks[i],mysegmask);

		// number of pixels in the image region
		numPix[i] = cvCountNonZero(masks[i]);

		// allocate a matrix for the feature vectors in this region
		D[i].set_size(dim,numPix[i]); D[i].fill(0.0);

		// update new segmentation mask
		cvSet(segmentMask,cvScalar((double) i),masks[i]);
	}

	
	/*** computing covariance descriptors ***/

	long int *ind = new long int[numRegions];
	int whichregion;
	
	for (int j=0;j<numRegions;j++)
		ind[j] = 0;
	im_itr<float>* vecitr[10];
	for (int i=0;i<dim;i++)
		vecitr[i] = new im_itr<float>(images[i]);
	im_itr<uchar> segitr(segmentMask);
	while (!segitr)
	{
		whichregion = int (*segitr);
		for (int i=0;i<dim;i++)
		{
			D[whichregion](i,ind[whichregion]) = **vecitr[i]; 
			++(*vecitr[i]); 
		}
		++ind[whichregion]; ++segitr;
	}

	for (int j=0;j<numRegions;j++)
		covdesc[j].ComputeDescriptor(D[j]);

	/****************************************/


	// compute neighborhood graph
	MakeNhoodGraph();

	// compute geometric properties - moments & eccentricity
	ComputeGeometrics();




	/*** clean-up ***/
	cvReleaseImage(&mysegmask);
	delete[] ind;
	for (int i=0;i<numRegions;i++)
	{
		D[i].~vnl_matrix();
	}
	/****************/

}


/*************************************************************************************************/


void ImageDescriptor::ShowMask(int i)
{
	cvNamedWindow("mask");
	cvShowImage("mask",masks[i]);
	cvWaitKey();
	cvDestroyWindow("mask");
}

/*************************************************************************************************/


/*************************************************************************************************/


/*************************************************************************************************/
// DESTRUCTOR
ImageDescriptor::~ImageDescriptor()
{
	cvReleaseImage(&segmentMask);
	/*for (int i=0;i<numRegions;i++)
	{
		cvReleaseImage(&masks[i]);
	}*/
	//delete [] numPix;

}

