/*-------------------------------------------------------------------------------
AUTHOR : KEDAR PATWARDHAN
DATE : 19 Jan 2005
DESCRIPTION : THIS IS A ITERATOR TEMPLATE I BUILT TO STEP THROUGH PIXELS
OF THE ENTIRE IMAGE
-------------------------------------------------------------------------------*/

#ifndef __IMAGEITERATOR_H__
#define __IMAGEITERATOR_H__

#include "cv.h"
#include "highgui.h"
/*#include "cvaux.h"*/
#include "math.h"

template <class T>
class im_itr
{
/*************************************************************************************************/
	int x0, start_x, start_y, curr_x, curr_y;
	int dataStep;
	int max_y, max_x;
	int nChannels;

	T* data, *origData;
	T* pixValue;

/*************************************************************************************************/
	public:
/*************************************************************************************************/
		/* constructor */
		im_itr(IplImage* image, int x=0, int y=0, int offset_x=0, int offset_y=0)
		{
			set(image, x, y, offset_x, offset_y);
		}
/*************************************************************************************************/
		/* end of file ? */
		bool operator !() const 
		{ 
			return (curr_y < max_y) ; 
		}
/*************************************************************************************************/
		/* next pixel */
		im_itr& operator ++()
		{
			curr_x ++;
			if( curr_x >= max_x)
			{
				curr_x = x0;
				curr_y ++;
				data += dataStep;
			}
			return *this;
		}
/*************************************************************************************************/
		im_itr& operator +=(int n)
		{
			curr_x += n;
			if( curr_x >= max_x)
			{
				curr_x = x0;
				curr_y ++;
				data += dataStep;
			}
			return *this;
		}
/*************************************************************************************************/
		/* access pixels */
		T& operator*() 
		{ 
			return data[curr_x] ; 
		}
/*************************************************************************************************/
		const T operator*() const 
		{ 
			return data[curr_x] ; 
		}
/*************************************************************************************************/
		const T neighbor( int dx, int dy ) const
		{
			if((curr_x+dx)<0 || (curr_x+dx)>=max_x) 
			{
				dx=dy=0;
			}
			else if((curr_y+dy)<0 || (curr_y+dy)>=max_y) 
			{
				dy=dx=0;
			}

			return *(data + dy*dataStep + curr_x + dx) ;
		}
/*************************************************************************************************/
		im_itr& putNeighbor( int dx, int dy, T val )
		{
			if(!( (curr_x+dx)<0 || (curr_x+dx)>=max_x || (curr_y+dy)<0 || (curr_y+dy)>=max_y) )
			{
				*(data + dy*dataStep + curr_x + dx) = val;
			}
			return *this;
		}
/*************************************************************************************************/
		T* operator &() const 
		{ 
			return (data + curr_x) ; 
		}
/*************************************************************************************************/
		/* current pixel location */
		int column() const 
		{ 
			return curr_x / nChannels ; 
		}
/*************************************************************************************************/
		int row() const 
		{ 
			return curr_y ; 
		}
/*************************************************************************************************/
		/* reset iterator */
		im_itr& reset()
		{
			curr_x = start_x;
			curr_y = start_y;
			data = origData;
			return *this;
		}
/*************************************************************************************************/
		/* set iterator */
		im_itr& set(IplImage* image, int x=0, int y=0, int offset_x=0, int offset_y=0)
		{
			curr_x=x;
			curr_y=y;
			x0=0;

			data = reinterpret_cast< T* > (image->imageData);
			dataStep = image->widthStep / sizeof( T );

			max_y = image->height;
			if ((y + offset_y)>0 && (y + offset_y)<max_y) 
			{
				max_y = y+offset_y;
			}
			if (y < 0) 
			{
				curr_y = 0;
			}
			data += dataStep * curr_y;

			max_x = image->width;
			if ( (x + offset_x)>0 && (x + offset_x)<max_x) 
			{
				max_x = x+offset_x;
			}
			max_x *= image->nChannels;
			if (x > 0) 
			{
				x0 = x*image->nChannels;
			}
			curr_x = x0;

			nChannels = image->nChannels;
			pixValue = new T [nChannels];

			start_x = curr_x;
			start_y = curr_y;

			origData = data;

			return *this;
		}
/*************************************************************************************************/
};


#endif





