
#include "stdafx.h"
#include "imgmath.h"

//=================================================================
//gradient
//=================================================================
void imgGradient ( IplImage* img, IplImage* Ix, IplImage* Iy, IplImage* mask )
{
	CvRect iroi = cvGetImageROI( img );
	CvRect mroi;
	bool maskzero = ( mask == 0);
	if( maskzero ) 
	{
		mask = cvCreateImage( cvGetSize( img ), IPL_DEPTH_8U, img->nChannels );
		setOneMask( mask );
		mroi = cvGetImageROI(img);
	}
	else
	{
		mroi = cvGetImageROI(mask);
	}

	im_itr<float> im( img, iroi.x, iroi.y, iroi.width, iroi.height );
	im_itr<float> ix( Ix, iroi.x, iroi.y, iroi.width, iroi.height );
	im_itr<float> iy( Iy, iroi.x, iroi.y, iroi.width, iroi.height );
	im_itr<uchar> ms( mask, mroi.x, mroi.y, mroi.width, mroi.height );

	while( !im && !ix && !iy && !ms ) 
	{
		//*ix = (im.neighbor(1,0) - im.neighbor(0,0));
		//*iy = (im.neighbor(0,1) - im.neighbor(0,0));
		*ix = (im.neighbor(1,0) - im.neighbor(-1,0)) / 2;
		*iy = (im.neighbor(0,1) - im.neighbor(0,-1)) / 2;
		++im; ++ix; ++iy; ++ms;
	}

	if( maskzero ) 
	{
		cvReleaseImage( &mask );
	}
}
//=================================================================
//Mod Grad
//=================================================================
void modGrad( IplImage* Ax, IplImage* Ay, IplImage* Dst, float ep)
{
	im_itr<float> ax(Ax);
	im_itr<float> ay(Ay);
	im_itr<float> ds(Dst);

	while( (!ax) & (!ay) & (!ds) )
	{
		float val=0;
		val = sqrt ( pow((*ax), 2) + pow((*ay), 2) + ep );
		if( val < 0) 
		{
			val=0;
		}
		if( val > 1)
		{
			val=1;
		}
		*ds = val;
		++ds; ++ax; ++ay;
	}
}

IplImage* modGrad( IplImage* img, IplImage* mask, float ep)
{
	IplImage* modgrad = cvCreateImage( cvGetSize(img), IPL_DEPTH_32F, img->nChannels );
	IplImage* mx = cvCloneImage( modgrad );
	IplImage* my = cvCloneImage( modgrad );

	imgGradient( img, mx, my, mask );
	modGrad( mx, my, modgrad, ep );

	cvReleaseImage( &mx );
	cvReleaseImage( &my );

	return modgrad;
}
//=================================================================
//image sum
//=================================================================
double imgSum( IplImage* image, IplImage* mask )
{
	double sum=0;
	bool imColor = (image->nChannels > 1 );
	CvRect iroi = cvGetImageROI( image );
	im_itr<float> im( image, iroi.x, iroi.y, iroi.width, iroi.height );
	im_itr<uchar> ms( mask, iroi.x, iroi.y, iroi.width, iroi.height );

	while( !ms && !im ) 
	{
		if( *ms == 255 ) 
		{
			sum += (*im);
			if( imColor ) 
			{
				++im; sum += (*im);
				++im; sum += (*im);
			}
		}
		else if(imColor)
		{
			im+=2;
		}
		++ms;
		++im;
	}

	return sum;
}

//=================================================================
//laplacian
//=================================================================
void imgLaplacian ( IplImage* img, IplImage* Lap, IplImage* mask )
{
	CvRect iroi = cvGetImageROI( img );
	CvRect mroi;
	bool maskzero = ( mask == 0);
	if( maskzero )
	{
		mask = cvCreateImage( cvGetSize( img ), IPL_DEPTH_8U,
		img->nChannels );
		setOneMask( mask );
	}
	else
	{
		mroi = cvGetImageROI(mask);
	}

	im_itr<float> im( img, iroi.x, iroi.y, iroi.width, iroi.height );
	im_itr<float> L( Lap, iroi.x, iroi.y, iroi.width, iroi.height );
	im_itr<uchar> ms( mask, mroi.x, mroi.y, mroi.width, mroi.height );

	while( !im && !L && !ms ) 
	{
		if( *ms )
		{
			float fxx =
			im.neighbor(-1,0)+im.neighbor(1,0)-2*im.neighbor(0,0);
			float fyy =
			im.neighbor(-1,0)+im.neighbor(1,0)-2*im.neighbor(0,0);
			*L = fxx+fyy;
		}
		++ms; ++L; ++im;
	}

	if( maskzero ) 
	{
		cvReleaseImage( &mask );
	}
}
//=================================================================
//diffusion - 2D
//=================================================================
void diffuse2D(IplImage* src_im/*1 Ch, float*/, IplImage* difmask, int numiters, float dt)
{
	IplImage* src = 0;
	bool srcIsInt = false;
	int src_depth = src_im->depth;
	if( src_depth == IPL_DEPTH_8U )
	{
		srcIsInt = true;
		src = cvCreateImage(cvGetSize(src_im),IPL_DEPTH_32F,src_im->nChannels);
	}
	else
	{
		srcIsInt = false;
		src = src_im;
	}

	bool mskzero = (difmask==0);
	if(mskzero)
	{
		difmask = cvCreateImage( cvGetSize(src), IPL_DEPTH_8U, 1 );
		setOneMask(difmask);
	}

	IplImage* lap = cvCloneImage( src );
	IplImage* oldsrc = cvCloneImage(src);
	for( int itr=0; itr < numiters; ++itr )
	{
		imgLaplacian(oldsrc,lap,difmask);
		cvAddWeighted(oldsrc,1.0,lap,dt,0,src);
		cvCopy(src,oldsrc);
	}

	if( mskzero ) 
	{
		cvReleaseImage( &difmask );
	}
	cvReleaseImage( &lap );
	cvReleaseImage( &oldsrc );
	if( srcIsInt ) 
	{
		cvReleaseImage( &src );
	}
}

