
#ifndef __SEGMENTIMAGE_H__
#define __SEGMENTIMAGE_H__

#include <cstdio>
#include <cstdlib>
#include "seg_image.h"
#include "seg_misc.h"
#include "seg_pnmfile.h"
#include "seg_segment-image.h"

#include "imgmath.h"
#include "generalutils.h"
#include "utility.h"
#include "imagedescriptor.h"

////////////////////////////////////////////////////////////////////////////////////////////////////


#define SEG_SIGMA 1.0
#define SEG_K 500.0
#define SEG_MIN_SIZE 500

////////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////////
/// EFFICIENT GRAPH-BASED IMAGE SEGMENTATION by Pedro F. Felzenszwalb and Daniel P. Huttenlocher
/// http://people.cs.uchicago.edu/~pff/segment/
////////////////////////////////////////////////////////////////////////////////////////////////////
int GraphSegment(void)
{
	image<rgb> *input = loadPPM("inpimage.ppm");
	int num_ccs; 
	clock_t startTime = clock();
	image<rgb> *seg = segment_image(input, SEG_SIGMA, SEG_K, SEG_MIN_SIZE, &num_ccs); 
	clock_t stopTime = clock();
	savePPM(seg, "outimage.ppm");
	cout << "Number of segments = " << num_ccs;
	cout << " ;  Time taken = " << ((float) (stopTime - startTime))/CLOCKS_PER_SEC << " seconds " << endl;
	return num_ccs;
}
////////////////////////////////////////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDescriptor segmentimage(IplImage* im, int dims = 10)
{
	
	cvSaveImage("inpimage.ppm",im);
	int numSegs = GraphSegment();
	IplImage *seg = cvLoadImage("outimage.ppm");
	CvSize imsize = cvGetSize(seg);


	/// computing image features ///////////////////////////
	IplImage* imD = cvCreateImage(imsize,IPL_DEPTH_32F,3);
	convert8Uto32F	( im , imD);
	IplImage** vecs = new IplImage*[dims];
	for (int i=0;i<dims;i++)
		vecs[i] = cvCreateImage(imsize,IPL_DEPTH_32F,1);

	// RGB colors
	cvSplit(imD, vecs[0], vecs[1], vecs[2], 0);

	// Ix and Iy first order gradients
	imgGradient(imD, vecs[3], vecs[4]);
	
	// gradient magnitude & direction
	cvCartToPolar(vecs[3],vecs[4],vecs[6],vecs[5]);

	// 2nd order derivatives
	imgGradient(vecs[4],vecs[8],vecs[9]);	// Iyy
	imgGradient(vecs[3],vecs[7],vecs[8]);	// Ixx and Ixy
	

	// take absolute value of the gradients
	cvAbsDiffS(vecs[3],vecs[3],cvScalar(0.0)); // Ix
	cvAbsDiffS(vecs[4],vecs[4],cvScalar(0.0)); // Iy
	cvAbsDiffS(vecs[7],vecs[7],cvScalar(0.0)); // Ixx
	cvAbsDiffS(vecs[8],vecs[8],cvScalar(0.0)); // Ixy
	cvAbsDiffS(vecs[9],vecs[9],cvScalar(0.0)); // Iyy
	////////////////////////////////////////////////////////


	IplImage* segD = cvCreateImage(imsize,IPL_DEPTH_32F,3);
	convert8Uto32F	( seg , segD);

	IplImage *seg1 = cvCreateImage(imsize,IPL_DEPTH_32F,1);
	IplImage *seg2 = cvCreateImage(imsize,IPL_DEPTH_32F,1);
	IplImage *seg3 = cvCreateImage(imsize,IPL_DEPTH_32F,1);
	IplImage *segmain = cvCreateImage(imsize,IPL_DEPTH_32F,1);
	cvSplit(segD,seg1,seg2,seg3,0);
	cvAddWeighted(seg1,1000.0,seg2,1.0,0.0,segmain);
	cvAddWeighted(segmain,1000.0,seg3,1.0,0.0,segmain);




	ImageDescriptor imdesc(vecs, segmain, numSegs, dims);

	return imdesc;
}
////////////////////////////////////////////////////////////////////////////////////////////////////





#endif