
//	standard includes
#include "stdafx.h"
#include	"math.h"

//	my includes
#include "video_interface.h"


/*************************************************************************************************/
//------------------------------------------------------------------------------------------------
// PRIVATE FUNCTIONS
//------------------------------------------------------------------------------------------------
/*************************************************************************************************/




/*************************************************************************************************/
/*************************************************************************************************/
/*************************************************************************************************/






/*************************************************************************************************/
//------------------------------------------------------------------------------------------------
// PUBLIC FUNCTIONS
//------------------------------------------------------------------------------------------------
/*************************************************************************************************/
// CONSTRUCTOR
VideoInterface::VideoInterface(char* name, double resize) : resizeFactor(resize)
{
	videoname.assign(name);
	capture = cvCreateFileCapture( name );

	if (!capture)
	{
		cerr << "Error : Video file could not be opened!!!" << endl;
		throw(-1);
	}

	width = (int) cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_WIDTH);
	height = (int) cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_HEIGHT); 	
	maxFrames = (int) cvGetCaptureProperty(capture,CV_CAP_PROP_FRAME_COUNT);
	
	inpim = cvCreateImage(cvSize(width,height),IPL_DEPTH_8U,3);

	if (resizeFactor!=1.0)
	{
		resizeImage = true;
		imsize.width = (int) floor( resizeFactor * (float) width );
		imsize.height = (int) floor( resizeFactor * (float) height );
	}
	else
	{
		imsize.width = width;
		imsize.height = height;
	}

	im = cvCreateImage(imsize,IPL_DEPTH_8U,3);
}


/*************************************************************************************************/

// Query the next frame
void VideoInterface::QueryFrame()
{
	inpim = cvQueryFrame(capture);
	if (!inpim)
		cerr << "Error : Cannot read from file!! " << endl;

	if (resizeImage)
		cvResize(inpim,im,CV_INTER_CUBIC);
	else
		cvCopy(inpim,im);

}

/*************************************************************************************************/

// Query a particular frame number
void VideoInterface::QueryFrame(long int frameNumber)
{
	frameNumber = min(frameNumber,maxFrames);
	if ( cvGetCaptureProperty(capture,CV_CAP_PROP_POS_FRAMES) != frameNumber )
		cvSetCaptureProperty(capture,CV_CAP_PROP_POS_FRAMES,frameNumber);
	inpim = cvQueryFrame(capture);
	if (!inpim)
		cerr << "Error : Cannot read from file!! " << endl;
	
	if (resizeImage)
		cvResize(inpim,im,CV_INTER_CUBIC);
	else
		cvCopy(inpim,im);
}

/*************************************************************************************************/


/*************************************************************************************************/
// DESTRUCTOR
VideoInterface::~VideoInterface()
{
	cvReleaseCapture( &capture );
	cvReleaseImage( &inpim );
	cvReleaseImage( &im );
}

